import ConfigParser
import os, shutil, subprocess, shlex

from platform import node
from glob import iglob

from vogeler.exceptions import VogelerPluginException

class VogelerPlugin(object):
    compiled_plugin_file = '/tmp/vogeler-plugins.cfg'
    authorized_plugins = ()
    plugin_registry = {}
    def __init__(self, plugin_dir='/etc/vogeler/plugins'):
        print "Vogeler is parsing plugins"
        self.registered_plugins = {}
        self.plugin_dir = plugin_dir
        self._compile_plugins()

    def execute_plugin(self, plugin):
        if plugin in self.authorized_plugins:
            try:
                command = self.plugin_registry[plugin]['command']
                plugin_format = self.plugin_registry[plugin]['result_format']
                result = subprocess.Popen(shlex.split(command), stdout = subprocess.PIPE).communicate()
                return self.format_response(plugin, result, plugin_format)
            except:
                #raise VogelerPluginException("Unable to execute plugin: %s" % command)
                print "Unable to execute plugin: %s" % command
                pass
        else:
            print "Plugin %s not authorized. Ignoring" % plugin
            pass
            #raise VogelerPluginException("Plugin - %s - not authorized for this host" % plugin)

    def format_response(self, plugin, output, plugin_format):
        message = { 'syskey' : node(), plugin : output[0], 'format' : plugin_format }
        return message

    def _compile_plugins(self):
        try:
            cpf = open(self.compiled_plugin_file, 'w')
            header = "#This is a compiled vogeler plugin file. Please do not edit!!!\n"
            cpf.write(header)
            for filename in iglob(os.path.join(self.plugin_dir, '*.cfg')):
                shutil.copyfileobj(open(filename, 'r'), cpf)
            cpf.close()
            self._read_plugin_file()
        except:
            raise VogelerPluginException("Unable to compile plugin files")

    def _read_plugin_file(self):
        configobj = ConfigParser.SafeConfigParser()
        try:
            configobj.read(self.compiled_plugin_file)
            self._parse_plugin_file(config=configobj)
        except:
            raise VogelerPluginException("Unable to parse compiled plugin file")

    def _parse_plugin_file(self, config):
        plugins = config.sections()
        print "Found plugins: %s" % plugins
        for plugin in plugins:
            plugin_details = dict(config.items(plugin))
            try:
                self._register_plugin(plugin_details)
                print "Registering plugin: %s" % plugin_details
            except:
                raise VogelerPluginException("Unable to parse plugin: %s" % plugin)

        self._authorize_plugins()

    def _authorize_plugins(self):
        print "Authorizing registered plugins"
        try:
            self.authorized_plugins = tuple(self.plugin_registry.keys())
        except:
            raise VogelerPluginException("Unable to authorize plugins: %s" % self.plugin_registry.keys())

    def _register_plugin(self, plugin_details):
        plugin = plugin_details.pop("name")
        try:
            self.plugin_registry[plugin] = plugin_details
        except:
            raise VogelerPluginException("Unable to register plugin: %s" % plugin)

# vim: set ts=4 et sw=4 sts=4 sta filetype=python :



import urlparse
import vogeler.db.couch as couch

from vogeler.exceptions import VogelerPersistenceException

"""vogeler.persisistence is used like so:

import vogeler.persistence as engine
# Create or find and then use the system_records db
c = engine.create_engine('couch://127.0.0.1:5984/system_records')
c.create_db()
# create or find the nodename node
c.create('nodename')
# update the nodename node with the given attributes
package_list = some_shell_command_output
c.update('nodename', 'packages', package_list', 'output')
mylist = ['foo','bar','baz']
c.update('nodename', 'my_python_list', mylist, 'pylist')
mydict = {'foo' : 1, 'bar' : 2, 'baz' : 'shoe'}
c.update('nodename', 'my_python_dict', mydict, 'pydict')
myyaml = some_yaml_data
c.update('nodename', 'my_yaml_data', myyaml, 'yaml')
c.update('nodename', 'my_string', 'some sting data', 'string')
myxml = some_xml_data
c.update('nodename', 'my_raw_data', myxml, 'raw')
c.drop_db()
"""
def create_engine(dsn, **credentials):
    try:
        scheme, params = _parse_url(dsn)
        connect_string = "%s.VogelerStore(%s)" % (scheme, params)
        engine = eval(connect_string)
        return engine
    except:
        raise VogelerPersistenceException("Unable to connect to backend database: %s" % connect_string)

def _parse_url(url):
    parsed = urlparse.urlparse(url)
    s = parsed.scheme
    h, p = parsed.netloc.split(':')
    d = parsed.path.split("/")[1]
    params = "host='%s', port='%s', db='%s'" % (h, p, d)
    return (s, params)

# vim: set ts=4 et sw=4 sts=4 sta filetype=python :
